@@ -3,14 +3,14 @@ package Text::Xslate::PP::Method;
 use strict;
 use warnings;
 
+use Text::Xslate::Util qw($DEBUG);
+use Text::Xslate::PP::State;
+use Text::Xslate::PP::Type::Pair;
 
 use Scalar::Util ();
 use Carp         ();
 
 require Text::Xslate::PP;
-require Text::Xslate::PP::State;
-require Text::Xslate::PP::Type::Pair;
-
 if(!Text::Xslate::PP::_PP_ERROR_VERBOSE()) {
     our @CARP_NOT = qw(
         Text::Xslate::PP::Opcode
@@ -76,12 +76,6 @@ sub _array_reduce {
     return $x;
 }
 
-sub _array_merge {
-    my($array_ref, $value) = @_;
-    return $_st->bad_arg('merge') if @_ != 2;
-    return [ @{$array_ref}, ref($value) eq 'ARRAY' ? @{$value} : $value ];
-}
-
 sub _hash_size {
     my($hash_ref) = @_;
     return $_st->bad_arg('size') if @_ != 1;
@@ -109,38 +103,19 @@ sub _hash_kv {
     ];
 }
 
-sub _hash_merge {
-    my($hash_ref, $other_hash_ref) = @_;
-    $_st->bad_arg('merge') if @_ != 2;
+our %builtin_method = (
+    'array::size'    => \&_array_size,
+    'array::join'    => \&_array_join,
+    'array::reverse' => \&_array_reverse,
+    'array::sort'    => \&_array_sort,
+    'array::map'     => \&_array_map,
+    'array::reduce'  => \&_array_reduce,
 
-    return { %{$hash_ref}, %{$other_hash_ref} };
-}
-
-BEGIN {
-    our %builtin_method = (
-        'array::size'    => \&_array_size,
-        'array::join'    => \&_array_join,
-        'array::reverse' => \&_array_reverse,
-        'array::sort'    => \&_array_sort,
-        'array::map'     => \&_array_map,
-        'array::reduce'  => \&_array_reduce,
-        'array::merge'   => \&_array_merge,
-
-        'hash::size'     => \&_hash_size,
-        'hash::keys'     => \&_hash_keys,
-        'hash::values'   => \&_hash_values,
-        'hash::kv'       => \&_hash_kv,
-        'hash::merge'    => \&_hash_merge,
-    );
-}
-
-sub tx_register_builtin_methods {
-    my($hv) = @_;
-    our %builtin_method;
-    while(my($k, $v) = each %builtin_method) {
-        $hv->{$k} = $v;
-    }
-}
+    'hash::size'     => \&_hash_size,
+    'hash::keys'     => \&_hash_keys,
+    'hash::values'   => \&_hash_values,
+    'hash::kv'       => \&_hash_kv,
+);
 
 sub tx_methodcall {
     my($st, $context, $method, $invocant, @args) = @_;
@@ -157,7 +132,7 @@ sub tx_methodcall {
              :                             'nil::';
     my $fq_name = $type . $method;
 
-    if(my $body = $st->symbol->{$fq_name}){
+    if(my $body = $st->symbol->{$fq_name} || $builtin_method{$fq_name}){
         push @{ $st->{ SP } }, [ $invocant, @args ]; # re-pushmark
         local $_context = $context;
         return $st->proccall($body, $context);
